function bitDepth = ApproxBitdepth(mImage)
% ApproxBitdepth  Returns the minimum bit depth that can be used to represent the range of
% pixel intensities in an image.


%
% Usage:        bitDepth = ApproxBitdepth(mImage);
%
% Inputs:
%   mImage      Matrix representation of an image.
%
% Outputs:
%   bitDepth    Number of bits required to store each element of the mImage
%               matrix without any coding.
%
functionName = mfilename;
%% Input validation
if nargin < 1
    fprintf('Image matrix not supplied\n');
    return;
end
%% Calculate bidDepth
% Assume images are always stored as unsigned integers
% Validate non the less
if min(mImage(:)) < 0
    error('%s: Image matrix supplied has negative pixel intensities',functionName);
end
maxIntensity = max(mImage(:));

if ~isfloat(maxIntensity)
    
    maxIntensity = double(maxIntensity);
    
end

if maxIntensity <= 0
    
    error('Unable to approximate bitdepth as the range of intensity values is too small!');
    
end

bitDepth = ceil(log(maxIntensity)/log(2));