function mCosines = DCos( szSpace, nFrequencies, redundancy )%#codegen 
% DCos Generates a matrix who's columns are Discrete Cosine vectors.
%
% Returns discrete cosine vectors that belong to the Euclidean space of size szSpace.  The 
% default is to return a basis for the space.
%
% Usage             mCosines = DCos( szSpace, nFrequencies, redundancy );
%                   mCosines = DCos( szSpace, nFrequencies );
%                   mCosines = DCos( szSpace );
%
% Inputs:
%   szSpace         Size of the Euclidean space the vectors should belong to.
%   nFrequencies    Number of frequencies to use starting from \(0\).  If not specified will be
%                   the same as the size of the space.
%   redundancy      Redundancy of the dictionary, the default is \(1\) (basis).
%
% Outputs:
%   mCosines        Matrix who's columns are discrete cosine vectors.

% See   http://www.ncrg.aston.ac.uk/Projects/HNLApprox/
%       http://www.nonlinear-approx.info/

error(nargchk(1,3,nargin));

if nargin <2

    nFrequencies = szSpace;
    redundancy = 1;
    
elseif nargin < 3
    
    redundancy = 1;
      
end

%mCosines = zeros(szSpace,nFrequencies);
n = 1:nFrequencies;
k = 1:szSpace;
N = redundancy*szSpace;
mCosines = cos( pi*(2*k-1)'*((n-1)/(2*N)) );