function vImage = Flatten(cImage,blockHeight,blockWidth,vIXBlockEnd)

    % Takes an mxn matrix and returns a 1 x nPixels vector containing all
    % the folded pixels in mImage.
    %   Ignores any pixels in the last blockHeight rows of the any plain in the image
    % image in
    % columns greater than vIXBlockEnd x blockWidth. i.e. the new matrix
    % only includes folded blocks and not padded zero blocks.
    %   Assumes mImage can be exactly blocked as it is the product of
    % previous blocking routine.
    
    nColours = numel(cImage);
    mFlat = [];
    
    % For each colour plain store the pixels
    for iColour = 1:nColours
            
        % Get current number of rows
        %   nColumns is not required.
        % <<--
            nRows = size(cImage{iColour},1);    
        % -->>

        % Number of useful columns in the last row of mImage and mIndex.
        %   Any pixels in the last blockHeight rows extending beyond this
        % column contain zeros used as padding.
        % <<--
            nColumnsRemaining = vIXBlockEnd(iColour)*blockWidth;
        % -->>

        % Number of row of mImage with full columns
        % <<--
            nRowsFull = nRows - blockHeight;
        % -->>

        % Flatten the image to be blockHeight x N
        %  The size of N is not important, reshape calculates this internally,
        % it is not required by the rest of the routine.
        % <<--

        % First take all the full rows
            mRowsFull = cImage{iColour}(1:nRowsFull,:);

        % Flatten mRowsFull to height blockHeight
            mFlat = [mFlat reshape(mRowsFull,blockHeight,[])];

        % Get the pixels containing folded data in the last blockHeight rows of
        % mImage.
            mLastBlocks = cImage{iColour}(nRowsFull+1:end,1:nColumnsRemaining);

        % Place the last nColumnsRemaining at the end of mImageFlat
            mFlat = [mFlat, mLastBlocks];

        % -->>
    end
    
            
    % Flatten mImageFlat to be a row vector
    % <<--
    % : has the same effect as using reshape except it will always return a
    % column vector, therefore transpose.
    vImage = mFlat(:)';
    % -->>

end