function [ mFoldedImage, vNCnPlain, sRescale, iLastXBlock] = FoldIndexGrey(...
    sImageApprox,sBlockDims,publicKey,privateKey, rotate,sTuning)

% Folds supplied indexes into an image generated using a constant atom.
% The coefficient of the constant atom is the number of atoms chosen in the
% approximation of previous blocks.

% PrintWithStars([mfilename ,' Routine Called']);

if nargin < 3
    publicKey = 19;
end
if nargin < 4
    privateKey = 987654321;
end
if nargin < 5
    rotate = 1;
end

if nargin < 6
    sTuning.vTuningFactor = 1;
    sTuning.quantMethod = 'none';
    sTuning.scaleEmbeddedImage = 0;
    sTuning.displayImages = 0;
    sTuning.displayAnalysis = 0;
    sTuning.hostMagnitude = 0;
    sTuning.hideIndex = 1;
    sTuning.rescaleIndex = 1;
    sTuning.encodeIndex = 0;
    sTuning.intensityAdjustment = 0;
    sTuning.method = 1;
end

% Show some info
% < < - - #
    showInfo = true;
% - - > > #

% Set up the key structure
% < < - - #
    sKey.publicKey = publicKey;
    sKey.privateKey = privateKey;
    sKey.rotate = rotate;
% - - > > #

% Get the indexes and the # of indexes per block
% < < - - #
    sHidden = GetAllIndexes(sImageApprox.cIndex);
% - - > > #

% Include extra params to pass in a column vector
% <<--
%  nAtoms chosen per block
    sHidden.vExtra = sHidden.mNCnPerBlock(:);
% -->>

% Permute the stored N with the private key
%     sHidden.vExtra = RandPerm(sHidden.vExtra,sKey.privateKey);
% - - > > #

% Calculate folded image dimensions

% Calculate host image dimensions
%   Calculate new dimensions with this value removed then call an aditional
% routine to calculate split off the nCn per block that will be plain text
% and the nCn that will be the coeff of the one atom.
% Host Routine needs to output a vector vNCnPlain as well
% Split them after the rescaling routine below has been called to maintain
% intesity of the image.
% < < - - #
    [nYBlocksNew iLastXBlock lFree nBlocks] = ...
        CalculateNewDimensionsOneAtom(sImageApprox.nIndexes,...
        sImageApprox.nXBlocks,sBlockDims.indexWidth,sBlockDims.indexHeight);
% - - > > #
    
% Set up image host structure
% < < - - #
    sImageHost = SImageHost;
    sImageHost.blockWidth = sBlockDims.indexWidth;
    sImageHost.blockHeight = sBlockDims.indexHeight;
    sImageHost.nXBlocks = sImageApprox.nXBlocks;
    sImageHost.nYBlocks = nYBlocksNew;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
    sImageHost.z = sImageApprox.z;
    sImageHost.iLastXBlock = iLastXBlock;
    sImageHost.iLastX = iLastXBlock*sImageHost.blockWidth;
    sImageHost.x = sImageApprox.x;
    sImageHost.y = sImageHost.nYBlocks*sImageHost.blockWidth;
    % Initialize to zero
    %   Populated inside GenerateEmbeddedImageOneAtom
    % < < - - 
        sImageHost.mImageHost = zeros(sImageHost.y,sImageHost.x);
    % - - > >
% - - > > #   
   
% Split # of atoms chosen per block into two vectors
%   vNCn - elements will be used as the intensity of each host block
%   vNCnPlain - will be returned by routine, the extra elements of
% sHidden.vExtra that cannot be used as the intensity of the host blocks as
% their are not enough.
% <<--
    sHidden.vNCn = sHidden.vExtra(1:nBlocks);
    % These have now been rescaled which is unecessary
    vNCnPlain = sHidden.vExtra(nBlocks + 1:end);
% -->>

% Rescale before hiding
%   The coefficients and nBlocks nCnPerBlock that are embedded should have the 
% same range of values.  The nCnPerBlock that are sent as plain text do not
% need to be rescaled here.
% < < - - #
    if sTuning.hideIndex && sTuning.rescaleIndex    
        sHidden = RescaleIndex(sHidden);    
    end
% - - > > #

% Add the extra padding now as rescaling has been applied
%   Pad vHiddenCoefficients with zeros to accomodate lFree.  If we were not
% using an orthogonal basis to hide the coefficients then we would
% distribute this space over all the blocks to avoid the calculation of the
% projector being ill posed, as the proctor will be orthogonal this step is
% unecessary.
% < < - - #
    sHidden.vICn = [sHidden.vICn; zeros(lFree,1)];
% - - > > #

% Calculate the embedded image
% < < - - #
    start = clock;
    sImageEmbedded = GenerateEmbeddedImageOneAtom(sImageHost,sHidden,sKey);
    elapsedTime = etime(clock,start);
% - - > > #

if showInfo    
%     PrintWithStars('Folding');
    fprintf('Folding Indexes:- %.2f seconds\n',elapsedTime);
end

% Fold the image
% < < - - #
    mFoldedImage = sImageEmbedded.mImageEmbedded;
% - - > > #

% Return the scaling params
%   Required for reconstruction
% < < - - #
    sRescale.minV = sHidden.minV;
    sRescale.maxV = sHidden.maxV;
    sRescale.rescaled = sHidden.rescaled;

end

% Start of sub functions########################################################
function [nYBlocks, iXBlockEnd, lFree, nBlocks] = ...
    CalculateNewDimensionsOneAtom(nCoefficients, nXBlocks, blockWidth, ...
    blockHeight)

    nPixelsBlock = blockWidth*blockHeight;
    nBlocks = ceil(nCoefficients/(nPixelsBlock-1));   
    nCoefficientsWithAtom = nCoefficients + nBlocks;
    lFree = (nBlocks*(nPixelsBlock)) - nCoefficientsWithAtom;
    % Make the image width as close from below to the original as possible
%     nXBlocks = floor(x/blockWidth);
%     nXBlocks = x/blockWidth;
    nYBlocks = ceil(nBlocks/nXBlocks);
    iXBlockEnd = nXBlocks - (nYBlocks*nXBlocks - nBlocks);
    %AdjustedSR = nPixels/(nCoefficientsWithAtom+lFree);

end


function sCn = GetAllIndexes(cIndex)
    

    sCn.mNCnPerBlock = cellfun(@numel,cIndex);

    % Scan left to right top to bottom to be consistent with recovery
    % routines
    cIndex = cIndex';
    sCn.vICn = [cIndex{:}]';
    
end

% For DPCM
%{
function sCn = GetAllIndexes(cIndex,nCoefficients)

    sCn.vICn = zeros(nCoefficients,1);
    [nRows nColumns] = size(cIndex);
    sCn.mNCnPerBlock = zeros(nRows,nColumns);

    iHidden = 1;
    rowIncrement = 1;
    iXBlockStart = 1;
    % Needed as we change the scan order for each row
    nXBlocksLoop = nColumns;
    %% Scan in the order we will apply the DPCM
    for i = 1:nRows

        for j = iXBlockStart:rowIncrement:nXBlocksLoop

%             nCn = numel(cIndex{i,j}.vIMDictionary2D);        
nCn = numel(cIndex{i,j});    
            %sCn.vCn(iHidden:iHidden+nCn-1) = sImage.cCoefficients{i,j};
%             sCn.vICn(iHidden:iHidden+nCn-1) = cIndex{i,j}.vIMDictionary2D;
sCn.vICn(iHidden:iHidden+nCn-1) = cIndex{i,j};
            sCn.mNCnPerBlock(i,j) = nCn;
            iHidden = iHidden + nCn;

        end
        temp = iXBlockStart;
        iXBlockStart = nXBlocksLoop;
        nXBlocksLoop = temp;
        rowIncrement = rowIncrement * -1;
    end
    %sCn.vCn = sCn.vCn(1:iHidden-1);
    sCn.vICn = sCn.vICn(1:iHidden-1);    

end
%}

function sHidden = RescaleIndex(sHidden)

    % Rescales the coefficients, the indexes, the number of atoms chosen
    % per block and the number of x and y blocks to have the same range
    % of values.
    % Use the maximum of the three ranges
    % The parameters will be stored as
    % [coefficients, indexes, extra]
    % Therefore recaled = 1 means the coefficients have been rescaled and
    % vMax1 refers to the indexes and vMax2 to the extra params etc.
    
    % Were the coefficients rescaled
    % <<--
    % Default to zero to check for errors
    sHidden.rescaled = 0;
    % -->>    
    
    % Max, min and range of the coefficients
    % <<--
    minICn = min(sHidden.vICn);
    maxICn = max(sHidden.vICn);
    rangeICn = maxICn-minICn;
    % -->>      
    
    % Max, min and range of the number of atoms required for each block and
    % the number or y and x blocks
    % <<--
    minVNCn = min(sHidden.vNCn);
    maxVNCn = max(sHidden.vNCn);
    rangeVNCn = maxVNCn-minVNCn;
    % -->>     
    
    
    % Rescale using integer rescaling params
    % Inefficient method, easy to follow
    % <<--
    % If the range used by the coefficients is greatest
    if rangeICn > rangeVNCn
        
        % Rescale the extra params
        [sHidden.vNCn sHidden.minV sHidden.maxV] = Rescale(...
            sHidden.vNCn,minICn,maxICn);
        
        % Needed for recovery
        sHidden.rescaled = 1;
        
    else
           
        % Rescale the indexes
        [sHidden.vICn sHidden.minV sHidden.maxV] = Rescale(...
            sHidden.vICn,minVNCn,maxVNCn);
        
        % Needed for recovery
        sHidden.rescaled = 2;
        
    end
   
    % -->>
        

end