function sImageApprox = ImageApproxSCEIFS(imagePath,error,approach,algorithm,...
    sBlockDims,sApproxDetails,intensityAdjustment)

% Wrapper around MImageApproximation.

% ApproxType
% 1) - Grey level image, 1 index and 1 coeff plains
% 2) - RGB one plain at a time, 3 index and 3 coeff plains
% 3) - Ml, 1 index and 3 coeff plains

% Should not be used apart from in testing
% < < - - #
    if nargin < 8
        intensityAdjustment = 0;
    end
% - - > > #


% Load the image
     sImage = LoadSImageRaw(imagePath,'uint16');
% - - > > #

% Set up the dictionary
    sDictionary = Dictionary(sApproxDetails.sDetails,...
        sBlockDims.approxHeight,1);
% - - > > #


% Is it RGB
% < < - - #
    rGB = (size(sImage.mImage,3) == 3);
% - - > > #

% Check if processing RGB seperatly
% < < - - #
    if rGB && ~strcmp(algorithm,'OMPMl2DW') && ~strcmp(algorithm,'OMP2DMlW')
        
        % Approximate the image one colour plane at a time
        % < < - - #
            [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
                nZeroBlocks] = RGBImageApproxSCEIFS(sImage.mImage,error,...
                approach,algorithm,blockWidth,blockHeight,sDictionary,...
                intensityAdjustment);
        % - - > > #        
        [nYBlocks nXBlocks]  = size(cCoefficients{1});
        nIndexes = nCoefficients;
        approxType = 2;
    else
        
        % Approximate the image as a whole, valid for grey images and RGB
        % using Ml
        % < < - - #
            [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
                nZeroBlocks] = MImageApproxSCEIFS(sImage.mImage,error,...
                approach,algorithm,...
                blockWidth,blockHeight,sDictionary,intensityAdjustment);
        % - - > > #
        [nYBlocks nXBlocks]  = size(cCoefficients);
        
        % If grey
        if sImage.z == 1
            nIndexes = nCoefficients;
            approxType = 1;
        % If RGB
        elseif sImage.z == 3
            nIndexes = nCoefficients/3;
            approxType = 3;
        end
        
        
    end
% - - > > #




% Initialize and populate the approximation structure
% < < - - #
    sImageApprox = SImageApprox;
    sImageApprox.method = algorithm;
    sImageApprox.mImage = mImageResized;
    sImageApprox.mImageApprox= mImageApprox;
    sImageApprox.cCoefficients = cCoefficients;
    sImageApprox.cIndex = cIndex;
    sImageApprox.nZeroBlocks = nZeroBlocks;
    sImageApprox.nCoefficients = nCoefficients;
    sImageApprox.nIndexes = nIndexes;
    [sImageApprox.y sImageApprox.x sImageApprox.z] = size(mImageResized);
    sImageApprox.blockWidth = blockWidth;
    sImageApprox.blockHeight = blockHeight;
    sImageApprox.nYBlocks = nYBlocks;
    sImageApprox.nXBlocks = nXBlocks;
    sImageApprox.approxType = approxType;
    
    % Copy remaining fields from image structure
    % < < - - 
        sImageApprox.path = sImage.path;
        sImageApprox.bitDepth = sImage.bitDepth;
        sImageApprox.actualBitDepth = sImage.actualBitDepth;
        sImageApprox.maxIntensity =  sImage.maxIntensity;
    % - - > >
% - - > > #