function [mImage bitDepth colourType map] = LoadMImage(imagePath)

% Loads an image from its path.
%   Converts indexed images to RGB or grey.
map = [];

%% Check if file exists
if exist(imagePath,'file') ~= 2
       
    error('%s not found in current directory or on MATLAB search path!',imagePath);
    
end
%% Assume its an image if imfinfo doesn't throw an exception
try
    
    sInfo = imfinfo(imagePath);
    
catch ME
    
    error('%s does not appear to be an image',imagePath);
    
end

% sImage.details = 'Image Structure';

if ~isfield(sInfo,'ColorType')
    error('%s: Illegal ColorType!',mfilename);
end
if isfield(sInfo,'BitDepth')
    bitDepth = sInfo.BitDepth;
else
    bitDepth = 0;
end
%% Check what type of image and load into structure, only grey, indexed and colorscale allowed
colourType = sInfo.ColorType;
if strcmp(colourType,'grayscale')
    mImage = imread(imagePath);
elseif strcmp(colourType,'truecolor')
    mImage = imread(imagePath);   
elseif strcmp(colourType,'indexed')    
    [mImage map] = imread(imagePath);
    % Convert to RGB
    % < < - - #
        mImage = ind2rgb(mImage,map);
    % - - > > #
else
    error('%s has unknown colour type!',imagePath);
end   
%% Populate the rest of the sImage structure
% sImage.path = imagePath;
% sImage.colourType = colourType;