function [mImageApprox mImageResized cCoefficients cIndex vNCoeff...
    vNZeroBlocks] = RGBImageApproxSCEIFS(mImage,error,approach,algorithm,...
    blockWidth,blockHeight,sDictionary,intensityAdjustment)

% Wrapper around MImageApproximation. Loads the image into a matrix and
% processes each colour plane indipendantly

% Should not be used apart from in testing
% < < - - #
    if nargin < 8
        intensityAdjustment = 0;
    end
% - - > > #

% % Load the image into a matrix
% % < < - - #
%     mImage = LoadMImage(imagePath);
% % - - > > #

% Dimensions of loaded image
% < < - - #
    nColours = size(mImage,3);
% - - > > #

% Check if RGB
% < < - - #
    if nColours ~= 3
        error('Supplies Image not RGB');
    end
% - - > > #

% Initialize return structures
% < < - - #
    % At this stage do not know what size the image will be
    cImageApprox = cell(1,1,nColours);
    cImageResized = cell(1,1,nColours);
    
    cCoefficients = cell(1,nColours);
    cIndex = cell(1,nColours);
    vNCoeff = zeros(1,nColours);
    vNZeroBlocks = zeros(1,nColours);
% - - > > #

% Repeat for each colour plane
% < < - - #
    for iColour = 1:nColours
        
        % Approximate the image
        % < < - -
            [cImageApprox{iColour} cImageResized{iColour}...
                cCoefficients{iColour} cIndex{iColour} vNCoeff(iColour) ...
                vNZeroBlocks(iColour)] = MImageApproxSCEIFS(...
                mImage(:,:,iColour),error,approach,algorithm,...
                blockWidth,blockHeight,sDictionary,intensityAdjustment);
        % - - > >

    end
% - - > > #

% Now dimensions are known copy cells to images
% < < - - #
    mImageApprox = cell2mat(cImageApprox);
    mImageResized = cell2mat(cImageResized);
% - - > > #

% Report on the quality of approximation
% < < - - #
%     nPixels = numel(mImageResized);
%     SR = nPixels/sum(vNCoeff);
%     maxIntensity = (2^ApproxBitdepth(mImageResized))-1;
%     PSNR = CalcPSNR(mImageResized,mImageApprox,maxIntensity);
%     window = fspecial('gaussian', 11, 1.5);	
%     K(1) = 0.01;					
%     K(2) = 0.03;					
%     SSIM = ssim(mImageResized,mImageApprox, K,window,maxIntensity);
%     fprintf('Processing supplied image matrix using %s in blocks of %i x %i\n',...
%         algorithm,blockHeight,blockWidth);
%     fprintf('The image has an approximate maximum intensity of %i\n',...
%         max(mImageResized(:)));
%     fprintf('SR of %g, achieved for PSNR:%.2fdB and SSIM:%.2f \n',SR,PSNR,SSIM);     
% - - > > #
