function [vImage vPlainText vNCnPlain approxType meanAdj]= ReshapeToRGBE(...
    mImageRGB,maxIntensity)

% Reverses ResizeRGB
% Takes a 3 dimensional array of pixel intensities representing an RGB
% image and reshapes into a 1D array by reversing the steps in ResizeRGBF.


% Reshape mImageRGB to be a 1d vector
%   Exact reverse of last step in ResizeRGB.
% <<--
    vImage = reshape(mImageRGB,1,[]);
% -->>

% Read approx type
%   Determins reconstruction method and # sPlainText params
    approxType = vImage(end-1);
% - - > > #

% Number of plain text parameters used for sPlainText
%   This is fixed
% <<--
    if approxType == 1
        nSPlainText = 39;
    elseif approxType == 2
        nSPlainText = 99;
    elseif approxType == 3
        nSPlainText = 51;
    else
        error('Unknown approx type!');
    end
% -->>

% Number of plain text parameters in total
%   This is the last value in vImage
% <<--% Flatten the image.

% Extract plain text
    multNPlainText = vImage(end-3);
    diffNPlainText = vImage(end-2);
    nPlainText = multNPlainText * maxIntensity + diffNPlainText;
% -->>

% Length of vNCnPlain
% <<--
    nVNCnPlain = nPlainText - nSPlainText;
% -->>

% Retrieve the padding
%   The last but 4 value
% <<--
    padding = vImage(end-4);
% -->>

% Retrieve the plain text mean adjustment
    meanAdj = vImage(end);
% - - > > #

% Retrieve vPlainText
    % Retrieve all values except the padding and nPlainText
    vPlainText = vImage(end-nPlainText+1:end-nVNCnPlain);
% -->>

% Retrieve vNCnPlain
    % -5 to ignore padding and multNPlainText, diffNPlainText, approxType
    % and plain text mean adjustment.
    vNCnPlain = vImage(end-nVNCnPlain+1:end-5);
% -->>

% Remove the padding and plain text parameters
% <<--
    nToRemove = padding + nPlainText;
    
    vImage = vImage(1:end-nToRemove);    
% -->>

% Resize vImage to the same dimensions as the folded image including
% padding if necessary

    % Number of column blocks in the original folded image is the same as in
    % mImageRGB
%     nXBlocks = size(mImageRGB,2)/blockWidth;
    
    % Resize
%     mImageGrey = Expand(vImage,blockHeight,blockWidth, nXBlocks);    
% - - > > #

end 

%{
function sPlainText = RetrievePlainText(vImage,nFields)

    % Retrieves the plain text parameters from the end of vImage.
    
    
    % Store the plain text variables
    % <<--
        vPlainText = vImage(end-nFields+1:end);
    % -->>
    
    
    % Populate sPlainText
    % <<--
    
        sPlainText.paddingCn = vPlainText(1);
        sPlainText.maxIntensity = vPlainText(2);
        sPlainText.nYBlocks = vPlainText(3);
        sPlainText.nXBlocks = vPlainText(4);
        sPlainText.minV1 = vPlainText(5);
        sPlainText.maxV1 = vPlainText(6);
        sPlainText.minV2 = vPlainText(7);
        sPlainText.maxV2 = vPlainText(8);
        sPlainText.rescaled = vPlainText(9);
        sPlainText.padding = vPlainText(10);
        sPlainText.multm = vPlainText(11);
        sPlainText.diffm = vPlainText(12);
        sPlainText.multM = vPlainText(13);
        sPlainText.diffM = vPlainText(14);       
        
        
    % -->>

end
%}

function nCn = GetNCN(nPixels,sPlainText)

    % Calculate the number of coefficients in the approximation from the
    % amount of padding included in the folded image.    
    
    % # of blocks in the original image
    % <<--
        nBlocks = sPlainText.nYBlocks*sPlainText.nXBlocks;
    % -->>
    
    % Get nCn
    % <<--
        %   Reverse sPlainText.paddingCn = nPixels - 2*nCn - nBlocks     
        nCn = (nPixels - sPlainText.paddingCn - nBlocks)/2;
    % -->>

end


%{
function mImageGrey = Expand(vImage, blockHeight, blockWidth, nXBlocks)

    % Reshape the image vector back to image cells
    
    %the 2d matrix representing the
    % original grey scale folded image.
    
    % Number of pixels in a block of the folded image
    % <<--
        nPixelsBlock = blockHeight*blockWidth;
    % -->>
    
    % # of blocks containing folded info
    % <<--
        nFoldedBlocks = ceil(numel(vImage)/nPixelsBlock);
    % -->>
    
    % # of rows and row blocks in original folded image
    % <<-
        nRowBlocks = ceil(nFoldedBlocks/nXBlocks);
        nRows = nRowBlocks * blockHeight;
    % -->>
    
    % # of columns and column blocks in original folded image
    % <--
        % Presently the same as the number of columns in the folded image        
        nColumnBlocks = nXBlocks;
        nColumns = nColumnBlocks * blockWidth;
    % -->>
    
    
    % Column index of last folded block in the last row of the folded image
    % <<--
    
        % # of blocks in the folded image
        nBlocks = nRowBlocks*nColumnBlocks;
        
        % # of blocks containing padding, i.e. added to get a folded image
        % of the correct dimentions.
        nPaddedBlocks = nBlocks - nFoldedBlocks;
        
        % Column index of the last block containing folded info.
        iLastXBlock = nColumnBlocks - nPaddedBlocks;
        
    % -->>
    

    % Initialize the 2d folded image mImageGrey
    % <<--
        mImageGrey = zeros(nRows,nColumns);
    % -->>
    
    
    % Populate the last row of mImageGrey
    % <<--
    
        % # of pixels in the last row of folded blocks
        %   Reversing the procedure in Flatten, i.e. first take the last n
        % elelments of vImage, reshape and place in the last row of mImageGrey
        % <<--
            nPixelsLastRow = iLastXBlock*nPixelsBlock;
        % -->>
        
        % Place pixels in temp vector
        vTemp = vImage(end-nPixelsLastRow+1:end);
        
        % Reshape vTemp to original dimensions.
        mTemp = reshape(vTemp,blockHeight,[]);
        
        % vTemp variable name will be used again so clear it
        clear vTemp;
        
        % Populate the last row of mImageGrey
        % <<--            
            mImageGrey(end-blockHeight+1:end,1:size(mTemp,2)) = mTemp;
        % -->>
        
        % mTemp variable name will be used again so clear it
        clear mTemp;
        
    % -->>
    
    
    % Poplate the full rows of mImageGrey
    % <<--
    
        % Get the remaining pixels from vImage
        vTemp = vImage(1:end-nPixelsLastRow);
        
        % Reshape to the original dimentions
        % <<--
            % For simplicity use [] as the number of columns is known, easier
            % to interpret.
            mTemp = reshape(vTemp,[],nColumns);
        % -->>
        
        % Populate the full rows of mImageGrey
        % <<--
            mImageGrey(1:size(mTemp,1),1:nColumns) = mTemp;
        % -->>

    % -->>
    
end
%}