function sImageApprox = SImageApproxSCEIFS(imagePath,PSNR,approach,algorithm,...
    sApproxDetails,sBlockDims,intensityAdjustment)

% Wrapper around MImageApproximation.

% ApproxType
% 1) - Grey level image, 1 index and 1 coeff plains
% 2) - RGB one plain at a time, 3 index and 3 coeff plains
% 3) - Ml, 1 index and 3 coeff plains

if nargin < 6
    % Default block processing dimentions
        sBlockDims.approxHeight = 8;
        sBlockDims.approxWidth = 8;
        sBlockDims.indexHeight = 8;
        sBlockDims.indexWidth = 8;
    % - - > > #
end

% Should not be used apart from in testing
% < < - - #
    if nargin < 7
        intensityAdjustment = 0;
    end
% - - > > #


% Load the image
     sImage = LoadSImageRaw(imagePath,'uint16');
% - - > > #

% Time the approximation
    startTime = clock;

% Set up the dictionary
    sDictionary = Dictionary(sApproxDetails.sDetails,...
        sBlockDims.approxHeight,1);
% - - > > #


% Is it RGB
% < < - - #
    rGB = (size(sImage.mImage,3) == 3);
% - - > > #

% Check if processing RGB seperatly
% < < - - #
    if rGB && ~strcmp(algorithm,'OMP2DMlW') && ~strcmp(algorithm,'OMPMlSepW')
                
        PrintWithStars('RGBImageApproxSCEIFS Routine called');
        % Approximate the image one colour plane at a time
        % < < - - #
            [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
                nZeroBlocks] = RGBImageApproxSCEIFS(sImage.mImage,PSNR,...
                approach,algorithm,sBlockDims.approxWidth,...
                sBlockDims.approxHeight,sDictionary,...
                intensityAdjustment);
        % - - > > #        
        [nYBlocks nXBlocks]  = size(cCoefficients{1});
        nIndexes = nCoefficients;
        approxType = 2;
    else
        
        PrintWithStars('MImageApproxSCEIFS Routine called');
        
        % Approximate the image as a whole, valid for grey images and RGB
        % using Ml
        % < < - - #
            [mImageApprox mImageResized cCoefficients cIndex nCoefficients ...
                nZeroBlocks] = MImageApproxSCEIFS(sImage.mImage,PSNR,...
                approach,algorithm,sBlockDims.approxWidth,...
                sBlockDims.approxHeight,sDictionary,intensityAdjustment);
        % - - > > #
        [nYBlocks nXBlocks]  = size(cCoefficients);
        
        % If grey
        if sImage.z == 1
            nIndexes = nCoefficients;
            approxType = 1;
        % If RGB
        elseif sImage.z == 3
            nIndexes = nCoefficients/3;
            approxType = 3;
        end
        
        
    end
% - - > > #

% Finish timing approximation - exclude calculation of PSNR and ssim
    approxTime = etime(clock,startTime);
        
% Replace all zero block indexes (-1) with 1 as the cosine dictionary is
% being used
%     if numel(cIndex) == 3
%         for i = 1:3
%             cIndex{i} = cellfun(@(x) Replace(x,-1,1),cIndex{i},'UniformOutput',0);
%         end
%     else
%         cIndex = cellfun(@(x) Replace(x,-1,1),cIndex,'UniformOutput',0);
%     end
% - - > > #

% Initialize and populate the approximation structure
% < < - - #
    sImageApprox = SImageApprox;
    sImageApprox.method = algorithm;
    sImageApprox.mImage = mImageResized;
    sImageApprox.mImageApprox= mImageApprox;
    sImageApprox.cCoefficients = cCoefficients;
    sImageApprox.cIndex = cIndex;
    sImageApprox.nZeroBlocks = nZeroBlocks;
    sImageApprox.nCoefficients = nCoefficients;
    sImageApprox.nIndexes = nIndexes;
    [sImageApprox.y sImageApprox.x sImageApprox.z] = size(mImageResized);
    sImageApprox.blockWidth = sBlockDims.approxWidth;
    sImageApprox.blockHeight = sBlockDims.approxHeight;
    sImageApprox.nYBlocks = nYBlocks;
    sImageApprox.nXBlocks = nXBlocks;
    sImageApprox.approxType = approxType;
    sImageApprox.approxTime = approxTime;
    
    % Copy remaining fields from image structure
    % < < - - 
        sImageApprox.path = sImage.path;
        sImageApprox.bitDepth = sImage.bitDepth;
        sImageApprox.actualBitDepth = sImage.actualBitDepth;
        sImageApprox.maxIntensity =  sImage.maxIntensity;
    % - - > >
    
    % Initialize calculated fields
    
% Report on the quality of approximation
% < < - - #
    sImageApprox.nPixels = numel(sImageApprox.mImageApprox);
    sImageApprox.SR = sImageApprox.nPixels/sum(sImageApprox.nCoefficients);
    fprintf('Processing supplied image matrix using %s in blocks of %i x %i\n',...
        algorithm,sImageApprox.blockHeight,sImageApprox.blockWidth);
    fprintf('The image has an approximate maximum intensity of %i\n',...
        max(mImageResized(:)));
    fprintf('The SR of the approximation is %g\n',sImageApprox.SR);
    [sImageApprox.PSNR sImageApprox.SSIM] = PrintPSNRSSIM(...
        sImageApprox.mImage,sImageApprox.mImageApprox,...
        sImageApprox.maxIntensity);
% - - > > #