function sHidden = SeparateCellIndex(sImageBlocked, sKey)
% SeparateCellImage Used by ExpandImage to 1) separate the host and embedded image using 
% SplitOrthComp and 2) retrieve the embedded coefficients using GenerateEncryptionOperator.


    % Find location of last used block
        sImageBlocked.nBlocksUsed = ceil(sImageBlocked.nIndexes/(...
            sImageBlocked.nPixelsBlock-1));
        sImageBlocked.lFree = sImageBlocked.nBlocksUsed*(sImageBlocked.nPixelsBlock-1)...
            - sImageBlocked.nIndexes; % Is this needed here???
        sImageBlocked.iLastXBlock = sImageBlocked.nBlocksUsed - ...
            ((sImageBlocked.nYBlocks-1)*sImageBlocked.nXBlocks);
    % - - > > #

    nCnHidden = sImageBlocked.nPixelsBlock - 1;
    % Allocate memory for variables
    vCn = zeros(sImageBlocked.nPixelsBlock*sImageBlocked.nYBlocks*sImageBlocked.nXBlocks,1);
    %cRecoveredImage = cell(size(cIndex));
    cNoise = cell(sImageBlocked.nYBlocks,sImageBlocked.nXBlocks);
    % Atom representing the image space
    vAtom = ones(sImageBlocked.nPixelsBlock,1);
    vAtom = vAtom/norm(vAtom);
    % Projector onto vAtom space
    mPOrthSpace = ((1/norm(vAtom)^2)*vAtom)*vAtom';
    

% Set up random matrices
%   Performed here to decrease processing time.
%   The public matrix should change for each block.
    % Private key - fixed matrix initialized to the largest possible size.
        RngW(sKey.privateKey);
        mRandPrivate =  rand(nCnHidden,nCnHidden) - 0.5;      
    % - - > >
    
    % Public key - initialize seed here
        RngW(sKey.publicKey);
    % - - > >
% - - > >    
    
% mNoise = zeros(sImageBlocked.blockHeight,sImageBlocked.blockWidth);
    %% Process first nYBlocks - 1 Rows
    iVCn = 1;
    iVNCn = 1;
    for i = 1:sImageBlocked.nYBlocks - 1

        for j = 1:sImageBlocked.nXBlocks    
            
            mOrthSpace= GenEncryptOp(vAtom,nCnHidden,mRandPrivate,sKey.rotate...
                ,mPOrthSpace);

            % Retrieve the number of atoms chosen in each block
            %   Each value in cRecoveredImage{i,j} should be the same, the number of
            % atoms chosen for an approximated image block.  Therefore round 
            % the mean value of cRecoveredImage{i,j}
                nCn = (vAtom'*sImageBlocked.cIndexFolded{i,j}(:));
            
            % Retrieve the noise
                vNoise = sImageBlocked.cIndexFolded{i,j}(:) - nCn*vAtom;

            % Store the hidden # of atoms per block
                vNCn(iVNCn) = nCn;
                iVNCn = iVNCn + 1;
            
            % Recover image coefficients from the noise
                vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
                iVCn = iVCn + nCnHidden;

        end

    end
    %% Deal with the last row seperatly as their maybe empty blocks
    for j = 1:sImageBlocked.iLastXBlock
        
        mOrthSpace= GenEncryptOp(vAtom,nCnHidden,mRandPrivate,sKey.rotate,...
            mPOrthSpace);
        
        % Retrieve the number of atoms chosen in each block
        %   Each value in cRecoveredImage{i,j} should be the same, the number of
        % atoms chosen for an approximated image block.  Therefore round 
        % the mean value of cRecoveredImage{i,j}
            nCn = vAtom'*sImageBlocked.cIndexFolded{...
                sImageBlocked.nYBlocks,j}(:);
            
        % Retrieve the noise
            vNoise = sImageBlocked.cIndexFolded{sImageBlocked.nYBlocks,j}(:)...
                - nCn*vAtom;

        % Store the hidden # of atoms per block
            vNCn(iVNCn) = nCn;
            iVNCn = iVNCn + 1;

        % Recover image coefficients from the noise
            vCn(iVCn:iVCn+nCnHidden-1) = mOrthSpace'*vNoise;
            iVCn = iVCn + nCnHidden;

    end
    %% Reduce the size of vCn so it only contains coefficients and padding
    sHidden.vIndex = vCn(1:iVCn-1);
    sHidden.vNIndex = vNCn;
    %% Return the noise as a matrix
    for i = sImageBlocked.iLastXBlock+1:sImageBlocked.nXBlocks
        cNoise{sImageBlocked.nYBlocks,i} = zeros(sImageBlocked.blockHeight,sImageBlocked.blockWidth);
    end

end

% %% Begin sub functions
% function [ mOrth, vOrthComp] = SplitOrthComp(mBlock, mPOrthSpcde)
% 
%     %% Remove the noise with orthogonal projection
% %     [mOrthAtoms mRTemp] = qr(mImageSpace,0);
% %     clear RTemp;
% %     mPOrthSpace = mOrthAtoms*mOrthAtoms';
%     vBlock = mBlock(:);
%     vOrth = (mPOrthSpace*vBlock);
%     mOrth = zeros(size(mBlock));
%     mOrth(:) = vOrth;
%     %% Retrieve the noise
%     vOrthComp = vBlock - vOrth;
%     
% end

%% Begin sub functions
function [ mOrth, vOrthComp] = SplitOrthComp(mBlock, mPOrthSpace)

    %% Remove the noise with orthogonal projection
%   mOrthAtoms mRTemp] = qr(mImageSpace,0);
%     clear RTemp;
%     mPOrthSpace = mOrthAtoms*mOrthAtoms';
    vBlock = mBlock(:);
    vOrth = (mPOrthSpace*vBlock);
    mOrth = zeros(size(mBlock));
    mOrth(:) = vOrth;
    %% Retrieve the noise
    vOrthComp = vBlock - vOrth;
    
end
