function ShowFig3(mImageFolded,cImageFolded,mImageRecovered,recoveryText)

    % Displays figure 3 for the paper ...
    % mImageFolded and mImageRecovered should be of the correct uint format
    
    % Format
    % 1) Folded image mImageFolded at the top
    % 2) Three seperate folded channels cImage{i}
    % 3) Three channels R,G,B of mImageRecovered underneath
    % 4) mImageRecovered at the bottom
    
    % Get the dimensions of mImageRecovered
    %   This will be used to resize the axis of the folded images to have
    % suitable dimensions.
    % <<--
        [nRows nColumns nChannels] = size(mImageRecovered);
    % -->>        
    
    % Create figure and split into subplots
    %   subplots, only use the middle subplot of the top and bottom rows.
    %   Enlarge to half the screen size for display
    % <<--
    
        figure;
        
        % Get the screen dimensions
        % <<--
            vScreen = get(0,'screensize');
            yPixels = vScreen(4);
            xPixels = vScreen(3);
        % -->>
        
        % Calc figure dimensions
        %   1/2 the size of the display with minimum of 430 x 550
        % <<--        
            yPixelsNew = yPixels/2;
            xPixelsNew = xPixels/2;
            
            % Check that this is greater than min   
            % <<--
                if yPixelsNew > 430 && xPixelsNew > 550
 
                    % Get the position of the bottom corneer
                    %   This will be 1/4 of the screen dimensions
                    % <<--
                        yStart = yPixels/4;
                        xStart = xPixels/4;
                    % -->>
                    
                    % Enlarge the figure
                    % <<--
                        set(gcf,'position',[xStart,yStart,xPixelsNew,...
                            yPixelsNew]);
                    % -->>

                end
            % -->>
        % -->> 
        
        % Create subplots and select the middle of the first row.
        %   Row major not standard column major order
        % <<--
            % Get starting plot and number of row plots
            %   If RGB this will be the middle of the first row otherwise
            % it will be 1.
            % <<--
                if nChannels == 3
                    
                    iCurrentPlot = 2;
                    nRowsPlot = 4;
                    
                else
                    
                    iCurrentPlot = 1;
                    nRowsPlot = 3;
                    
                end
            % -->>
            nColumnsPlot = nChannels;
            h = subplot(nRowsPlot,nColumnsPlot,iCurrentPlot);
        % -->>
    % -->>
    
    % Display mImageFolded with dimensions
    % <<--
        DisplayFolded(mImageFolded,h,nRows,nColumns);
        [yFolded, xFolded, zFolded] = size(mImageFolded);
        title(['Folded Image: ' num2str(yFolded) 'x' num2str(xFolded) 'x' ...
            num2str(zFolded) ]);
    % -->>
    
    
%     iCurrentPlot = iCurrentPlot + 1;
    
    % Display folded channels [cImage{i};cIndex{i}]
    % <<--
        if nChannels == 3
            cChannelText = {'Red','Green','Blue'};
            % Move to the end of the row
            iCurrentPlot = iCurrentPlot + 1;
            
            for i = 1:nChannels
                % Move to the next plot
                iCurrentPlot = iCurrentPlot + 1;
                % Concatinate image and index arrays to get image
%                 mImage = [cImage{i};cIndex{i}];
                % Select subplot
                h = subplot(nRowsPlot,nColumnsPlot,iCurrentPlot);
                % Display the image
                DisplayFolded(cImageFolded{i},h,nRows,nColumns);
%                 [yFolded, xFolded, zFolded] = size(mImage);
                title([cChannelText{i} ' Channel Folded']);
%                     num2str(yFolded) 'x' num2str(xFolded) 'x' ...
%                     num2str(zFolded) ]);
            end
        else
             iCurrentPlot = iCurrentPlot + 1;
             h = subplot(nRowsPlot,nColumnsPlot,iCurrentPlot);
             DisplayFolded(cImageFolded{1},h,nRows,nColumns);
             title('Grey Image Folded');
        end
    % -->>
    
    
    %{
    % Display folded channels [cImage{i};cIndex{i}] with minumum
    % gap minRowsToSplit between cImage{i} and cIndex{i}
    % <<--
        % Get the images
        % <<--
            minRowsToSplit = 100;
            cImage = JoinImage(cImage,cIndex,minRowsToSplit,nChannels);
        % -->>
        
        % Display the images
        % <<--
            for i = 1:nChannels
                % Select subplot
                h = subplot(nRowsPlot,nColumnsPlot,2*nColumnsPlot+i);
                % Display the image
                DisplayFolded(cImage{i},h,nRows,nColumns);
            end
        % -->>
    % -->>
    %}
    
    % Display the channels of mImageRecovered
    % <<--   
%         [yFolded, xFolded, zFolded] = size(mImageRecovered);
        if nChannels == 3
            cChannelText = {'Red','Green','Blue'};
            for i = 1:nChannels
                % Move to the next plot
                iCurrentPlot = iCurrentPlot + 1;
                % Select the ith subplot on row 2
                subplot(nRowsPlot,nColumnsPlot,iCurrentPlot);
                % Display the colour channel
                imshow(mImageRecovered(:,:,i));
                title([cChannelText{i} ' Channel Recoverd']);% ...
%                     num2str(yFolded) 'x' num2str(xFolded) 'x' ...
%                     num2str(zFolded) ]);
%                 DisplayFolded(mImageRecovered(:,:,i),h,nRows,nColumns);
            end
        end
    % -->>
    
    % Display mImageRecovered on the midle of the bottom row
    %   Only display if RGB
    % <<--
        if nChannels == 3
            % Move to the next plot
            iCurrentPlot = iCurrentPlot + 2;
        else
            % Move to the next plot
            iCurrentPlot = iCurrentPlot + 1; 
        end
        [yFolded, xFolded, zFolded] = size(mImageRecovered);
        subplot(nRowsPlot,nColumnsPlot,iCurrentPlot);
        imshow(mImageRecovered);
        title({['Recoverd Image: ' ...
            num2str(yFolded) 'x' num2str(xFolded) 'x' ...
            num2str(zFolded) ], recoveryText});
        %end
    % -->>
    
end


function DisplayFolded(mImage,h,nRows,nColumns)

    % Selects the subplot refered to by axis handle h and then displays the
    % image mImage and resizes the axis to nRows x nColumns.
    
    % Select subplot specified by axis handle h
    % <<--
        subplot(h);
    % -->>
    
    % Display the image mImage
    % <<--
        imshow(mImage);
    % -->>
    
    % Resize the axis
    % <<--
        axis([0.5 nColumns+0.5 0.5 nRows+ 0.5]);
    % -->>

end

function cFolded = JoinImage(cImage,cIndex,minRowsToSplit,nChannels)

    % Returns a cell containing 3 images. Each new image is a concatination
    % of cImage and cIndex with a minimum gap of minNRowsToSplit rows
    % between them.
    
    % Get the number or rows in each new image without the gap
    % <<--
        vNRows = zeros(1,nChannels);
        for i = 1:nChannels
            vNRows(i) = size(cImage{i},1) + size(cIndex{i},1);
        end
    % -->>
    
    % Calculate the new number of rows in the split image
    %   This will be the same for each image.  The max number of rows
    % over all the new images + minRowsToSplit.
    % <<--
        nRowsNew = max(vNRows) + minRowsToSplit;
    % -->>
        
    % Calculate the gap for each image
    % <<--
        vGap = nRowsNew - vNRows;
    % -->>
    
    % Place each image in cFolded
    % <<--
        % Number of columns needed for padding
        nColumns = size(cImage{1},2);
        
        cFolded = cell(1,nChannels);
        for i = 1:nChannels
            % Concatinate 
            cFolded{i} = [cImage{i}; 2^16-1*ones(vGap(i),nColumns); ...
                cIndex{i}];
        end
    % -->>


end








            