/* Implementation of pursuit_functions.h */

#include "pursuit_functions.h"
#include "matrix_operations.h"
#include <math.h>
//#include <vector>
#include "mex.h"
#include <algorithm>

#define error mexErrMsgTxt

/* Calculation routines */
void CalcSepIndex(double *vISepAtoms,double *vIndDictColumns,double *vIndDictRows,
        double nAtomsColumn, ulng nIndexes)
{
    /* Caclulates the indexes of the 2D seperable atom from the indexes of 
     * seperable atoms 
     *  vISep = vIRow*nAtomsColumn + vIColumn
     */
    
    /* Using y <- alpha*x + y, make y = vIColumn */
    CopyDbl(vIndDictColumns,vISepAtoms,nIndexes);
    
    /* vISepAtoms = vIndDictRows*nAtomsColumn + vISepatoms */
    Matrix::VAdd(vISepAtoms, vIndDictRows, nAtomsColumn, nIndexes);     
}
        
void CalcIndexYAndX(ulng indexYX,ulng nColumnAtoms,ulng nRowAtoms,
        ulng &iColumnAtom,ulng &iRowAtom)
{
    /* Calculates index's need to add 1 to return to MATLAB */
    iRowAtom = static_cast<unsigned long>(floor(static_cast<double>(indexYX)/
            static_cast<double>(nColumnAtoms)));
    iColumnAtom = (indexYX - (iRowAtom*nRowAtoms));
}        

void ValidateIndex(double *ind, ulng nAtoms, ulng numInd)
{
    /* Check that the indexes in ind are positive and do not nAtoms.
     * Achiveved by sorting the array ind and checking that
     * the min element is greater than 0 and that the max element is less 
     * than or equal nAtoms.
     * The array ind and is avaliable in the MATLAB workspace. 
     * Copy before sorting to avoid altering MATLAB workspace variables.
     * Throw error if validation unsuccessful.
     */
    
    // Copy and sort ind array
    double *indToSort = new double[numInd];
    CopyDbl(ind, indToSort, numInd);
    std::sort(indToSort, indToSort + numInd);
    // Validate smallest element > 0 and largest element <= nAtoms
    unsigned long smallestElement = static_cast<unsigned long>(indToSort[0]);
    unsigned long largestElement = static_cast<unsigned long>(indToSort[numInd-1]);
    if( (smallestElement <= 0 ) || (largestElement > nAtoms))
    {
            error("Demanded index's are out of dictionary!!\n");
    } 
    
    // Free dynamicaly assigned array
    delete [] indToSort;
    indToSort = 0;
          
}