/* Implementation of my_blas.h */

#include "my_blas.h"

#include "mex.h"  // include before blas.h for definition of ptrdiff_t
#include "blas.h"


void MatrixVectorMultBLAS(dbl *matrix, dbl *vector, dbl *returnVector, 
	ulng nRows, ulng nColumns, dbl alpha, dbl beta, char chn)
{
    /* Remember n is the number of columns in the matrix so it wil be one more than the index in 
     * an array of the same size
     */
    //dbl zero = 0, one = 1;
    ptrdiff_t incx = 1;
    //ptrdiff_t t1,t2;
    ptrdiff_t t1 = static_cast<ptrdiff_t>(nRows);
    ptrdiff_t t2 = static_cast<ptrdiff_t>(nColumns);
    dgemv(&chn, &t1, &t2, &alpha, matrix, &t1, vector, &incx, &beta, returnVector, &incx);
}

void ScaleVectorBLAS(dbl *v1, dbl alpha, ulng szVector)
{
    ptrdiff_t incx = 1;
    ptrdiff_t t1 = static_cast<ptrdiff_t>(szVector);
    dscal(&t1, &alpha, v1, &incx);   
}

dbl NormBLAS(dbl *atom, ulng szVector, ulng incx)
{
    ptrdiff_t t1 = static_cast<ptrdiff_t>(szVector);
    ptrdiff_t incx_t = static_cast<ptrdiff_t>(incx);
    return dnrm2(&t1,atom,&incx_t);
}

dbl InnerProductBLAS(dbl *v1, dbl *v2,  ulng m, 
	ulng incx, ulng incy)
{
    ptrdiff_t t1 = static_cast<ptrdiff_t>(m);
    ptrdiff_t incx_t = static_cast<ptrdiff_t>(incx);
    ptrdiff_t incy_t = static_cast<ptrdiff_t>(incy);
    return ddot(&t1, v1, &incx_t, v2, &incy_t);    
}

void AddVectorsBLAS(dbl *v1, dbl *v2, dbl alpha, ulng m, 
	ulng incx, ulng incy )
{
    ptrdiff_t incx_t = static_cast<ptrdiff_t>(incx);
    ptrdiff_t incy_t = static_cast<ptrdiff_t>(incy);
    ptrdiff_t t1 = static_cast<ptrdiff_t>(m);
    daxpy(&t1, &alpha, v1, &incx_t, v2, &incy_t);
}

void MatrixMatrixMultBLAS(double *m1, double *m2, double *mReturn, unsigned long m, unsigned long n,
	unsigned long k, char chn1, char chn2, double alpha, double beta)
{
    /* Remember n is the number of columns in the matrix so it wil be one more than the index in 
     * an array of the same size
     */
	// m is the number of rows of m1
	// n is the number of columns of m1
	// k is the number of columns of mReturn


	/* Always fixed */
	// The dimensions of m1 are an input to the function it is always m*n
	ptrdiff_t LDA = m;
	//ptrdiff_t LDB = n;
	// The number of columns of mReturn is an input to the function
	ptrdiff_t n_t = k;

	ptrdiff_t m_t, k_t;
	if (chn1 == 'T')
	{
		// If 1st matrix is not transposed then dimensions of m1 and op(m1) are switched
		m_t = n;
		k_t = m;
	}else{
		// If 1st matrix is not transposed then dimensions of m1 and op(m1) are the same
		m_t = m;
		k_t = n;
	}

	// If transposed then 1st dim is 2nd dim of return vector otherwise has to be number of columns in op(m1)
	ptrdiff_t LDB;
	if (chn2 == 'T')
	{
		LDB = k;
	}else{
		LDB = k_t;
	}

	// The 1st dimension of the return matrix is the same as the 1st dimension of op(m1)
	ptrdiff_t LDC = m_t;

    dgemm(&chn1,&chn2, &m_t, &n_t, &k_t, &alpha, m1, &LDA, m2, &LDB, &beta, mReturn, &LDC);
}

ulng MaxAbsBLAS(double *v, unsigned long m)
{
	ptrdiff_t incx = 1;
	ptrdiff_t m_t = static_cast<ptrdiff_t>(m);
	return static_cast<ulng>(idamax(&m_t, v, &incx));
}

void OuterBLAS(double *m1, double *v1, double *v2, double alpha, unsigned long m, unsigned long n)
{
    /* Remember n is the number of columns in the matrix so it wil be one more than the index in 
     * an array of the same size
     */
    ptrdiff_t incx = 1;
    ptrdiff_t t1,t2;
    t1 = static_cast<ptrdiff_t>(m);
    t2 = static_cast<ptrdiff_t>(n);
    dger(&t1,&t2,&alpha,v1,&incx,v2,&incx,m1,&t1);
}