/* Implementation of OMPMlSep_Funs */

#include "OMPMlSep_Funs.h"
#include "matrix_operations.h"
#include "pursuit_functions.h"
#include <math.h>
//#include "OMP_Funs.h"


void OMPMlSepN::ChooseAtom(dbl *mDictColumns,dbl *mDictRows,dbl *mResidule, 
        ulng nRows,ulng nColumns,ulng nColumnAtoms,ulng nRowAtoms,
        ulng nColours,dbl &maxValue,ulng &iMaxSepAtom,ulng &iColumnAtom,
        ulng &iRowAtom)
{         
    /* Choose the next coefficient
     * Perform inital matrix mult then loop through calculating the next mult
     * finding the max absolute value each time.
     * Requires mDictColumnsW to be pre weighted.
     */

    /* MATLAB
    % Calculate inner products for each plain
    %   Cannot use average due to subativity of abs
        cc=zeros(nColumnAtoms,nRowAtoms);
        for iColour=1:nColours;
            cc = cc + vWeights(iColour)*abs(mColumnDict'*...
                mResidule(:,:,iColour)*mRowDict);
        end
    % - - > >
     */
    

    /* Perform first matrix mult:- mTemp = mDictColumns'*mResidule */
    
    // Allocate space for temporary matrix
        ulng nColTemp = nColumns*nColours;
        ulng nElTemp = nColumnAtoms*nColTemp;
        dbl *mTemp = new dbl[nElTemp];
        
        // Initialize to zero
        //ZeroFillDbl(mTemp,nElTemp); //MOVE INTO MATRIXMATRIXMULT ??
        
        Matrix::MMMult(mDictColumns,mResidule,mTemp,nRows,nColumnAtoms,nColTemp,
            nColumnAtoms,'T','N');
        
    // - - > >
    
        
    /* Loop through each colour plain calculating the inner products */
        
        // Allocate space to store the coefficients for all plains
        // ? Will this avoid cache misses??
		//ulng nPixelsPlain = nRows*nColumns;
        ulng nCoeffPlain = nColumnAtoms*nRowAtoms;
        ulng nCoeff = nCoeffPlain*nColours;
        dbl *mCoeffAll = new dbl[nCoeff];


        // Index of first coefficient for current colour plain
        ulng iStartCoeff = 0;
		ulng iTemp = 0;

		// # of pixels in each plain of mTemp
		ulng nTempPlain = nColumnAtoms*nColumns;

        // Initialize to zero
        //ZeroFillDbl(mCoeffAll,nCoeff);  // MOVE INTO MATRIXMATRIXMULT

        // Loop through and calculate all the coefficients - is this better for the
        // cache check with cachegrind
        for (ulng iColour = 0; iColour < nColours; iColour++)
        {

            // Perform second matric mult for each colour plain
            // mCoeffPlain = mTemp*mDictRows;
            Matrix::MMMult(&mTemp[iTemp],mDictRows,&mCoeffAll[iStartCoeff],nColumnAtoms,
                    nColumns,nRowAtoms,nColumnAtoms,'N','N'); 

			// Calculate the absolute value
			//Matrix::VABS(&mCoeffAll[iStartCoeff],nCoeffPlain);

			// Then try alocating to a different array?? write into a different VABS function 

            // Position of next set of coefficints
            iStartCoeff += nCoeffPlain;
			iTemp += nTempPlain;

        }
        
    // - - > >
    
		// Take absolute value of all the coefficients
    //Matrix::VABS(mCoeffAll,nCoeff);  //  MOVE INSIDE LOOP ABOVE - PERFORM EVERYTHING IN LOOP.

for( ulng i = 0 ; i < nCoeff; ++i )
    {        
        mCoeffAll[i] = mCoeffAll[i] < 0.0 ? -mCoeffAll[i] : mCoeffAll[i];
    }



    // Free mTemp
    delete [] mTemp;
    mTemp = 0;

    
    // Use the first nCoeffPlain elements of vCoeffAll to sum all the coefficients
    // Saves more mem alloc + the first plain allready contains the coefficients
    // of that would be copied accross.
    
    // Index of first coefficient of the second colour plain
    iStartCoeff = nCoeffPlain;
    
    for (ulng iColour = 1; iColour < nColours; iColour++ )
    {
        
        // mCoeff(1:nCoeffPlain) = mCoeff(1:nCoeffPlain) + mCoeffAll(:,iStartCoeff:?)
        Matrix::VAdd(mCoeffAll,&mCoeffAll[iStartCoeff],1,nCoeffPlain);
        
        // Position of next set of coefficints
        iStartCoeff += nCoeffPlain;       

    }
    
    // Find the index of the max atom over all colour plains
	iMaxSepAtom = Matrix::MaxAbs(mCoeffAll, nCoeffPlain);
    
    // Get the max value - used to test convergence
    maxValue = mCoeffAll[iMaxSepAtom];
    
    // Free mCoeffAll
    delete [] mCoeffAll;
    mCoeffAll = 0;
    
    /* Get the index's in the seperate dictionaries */
    CalcIndexYAndX(iMaxSepAtom,nColumnAtoms,nRowAtoms,iColumnAtom,iRowAtom);    
}


//dbl OMPMlN::UpdateResidule(dbl *mResidule, dbl *mImage, dbl *vOrth, dbl *vWeights,
//	ulng nColours, ulng nPixelsPlain)
//{
//	/* Updates the residule and calculates the sum of the norms of each plain */
//
//		/* MATLAB
//		% Calculate component of residule in mOrth
//			for iColour = 1:nColours
//				mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
//					mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
//			end 
//		% - - > > #
//
//		% Update the residule
//			mResidule = mResidule - mRemove;
//		% - - > > #    
//
//        normResidule = sum(reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
//            nColours,1));
//		*/
//
//	/* Position in mImage and mResidule */
//	ulng iStartPlain = 0;
//	dbl normResidule = 0;
//	for (ulng iColour = 0; iColour < nColours; iColour++)
//	{
//		OMPN::UpdateResidulePlain(&mResidule[iStartPlain],&mImage[iStartPlain],vOrth,
//			vWeights[iColour],nPixelsPlain);
//
//		/* Calculate the norm of the residule of this colour plain */
//		 normResidule += Matrix::VNorm(&mResidule[iStartPlain],nPixelsPlain);
//
//		iStartPlain += nPixelsPlain;
//	}
//
//	return normResidule;
//}

dbl OMPMlN::UpdateResiduleW(dbl *mResidule, dbl *mImage, dbl *vOrth, dbl *vWeights,
	ulng nColours, ulng nPixelsPlain)
{
	/* Updates the residule and calculates the sum of the norms of each plain */
	// For pre weighted residule

		/* MATLAB
		% Calculate component of residule in mOrth
			for iColour = 1:nColours
				mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
					mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
			end 
		% - - > > #

		% Update the residule
			mResidule = mResidule - mRemove;
		% - - > > #    

        normResidule = sum(reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
            nColours,1));
		NOW SUM SQUARED ERROR
		*/

	/* Calculate inner products for whole image - vImage'*mOrth(:,iIter); */

	// Array to store inner products
	dbl *vInners = new dbl[nColours];

	// Calc inners
	Matrix::MVMult(mImage,vOrth,vInners,nPixelsPlain,nColours,nColours,'T');

	/* Loop through an update each plain of the residule */
	/* Position in mImage and mResidule */
	ulng iStartPlain = 0;
 	dbl ssResidule = 0;
	for (ulng iColour = 0; iColour < nColours; iColour++)
	{

		/* Remove the projection 
		mResidule = mResidule - innerProd*vOrth*/
		Matrix::VAdd(&mResidule[iStartPlain],vOrth,-vInners[iColour]*vWeights[iColour],nPixelsPlain);

		//OMPN::UpdateResidulePlain(&mResidule[iStartPlain],&mImage[iStartPlain],vOrth,
			//vWeights[iColour],nPixelsPlain);

		/* Calculate the norm of the residule of this colour plain */
		 ssResidule += Matrix::VNorm(&mResidule[iStartPlain],nPixelsPlain);

		iStartPlain += nPixelsPlain;
	}

    /* Calculate the norm of all the colour plains now the weights have been applied */
//     dbl ssResidule = Matrix::VNorm(mResidule,nPixelsPlain*nColours);
    
	/* Adjust to be the sum squared error */
	ssResidule = pow(ssResidule,2);

	delete [] vInners;
	vInners = 0;

	return ssResidule;
}


dbl OMPMlN::UpdateResidule(dbl *mResidule, dbl *mImage, dbl *vOrth,
	ulng nColours, ulng nPixelsPlain)
{
	/* Updates the residule and calculates the sum of the norms of each plain */

		/* MATLAB
		% Calculate component of residule in mOrth
			for iColour = 1:nColours
				mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
					mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
			end 
		% - - > > #

		% Update the residule
			mResidule = mResidule - mRemove;
		% - - > > #    

        normResidule = sum(reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
            nColours,1));
		NOW SUM SQUARED ERROR
		*/

	/* Calculate inner products for whole image - vImage'*mOrth(:,iIter); */

	// Array to store inner products
	dbl *vInners = new dbl[nColours];

	// Calc inners
	Matrix::MVMult(mImage,vOrth,vInners,nPixelsPlain,nColours,nColours,'T');

	/* Loop through an update each plain of the residule */
	/* Position in mImage and mResidule */
	ulng iStartPlain = 0;
	dbl ssResidule = 0;
	for (ulng iColour = 0; iColour < nColours; iColour++)
	{

		/* Remove the projection 
		mResidule = mResidule - innerProd*vOrth*/
		Matrix::VAdd(&mResidule[iStartPlain],vOrth,-vInners[iColour],nPixelsPlain);

		//OMPN::UpdateResidulePlain(&mResidule[iStartPlain],&mImage[iStartPlain],vOrth,
			//vWeights[iColour],nPixelsPlain);

		iStartPlain += nPixelsPlain;
	}

	/* Calculate the norm of the residule of this colour plain */
	ssResidule = Matrix::VNorm(mResidule,nPixelsPlain*nColours);

	/* Adjust to be the sum squared error */
	ssResidule = pow(ssResidule,2);

	delete [] vInners;
	vInners = 0;

	return ssResidule;
}

void OMPMlN::UpdateApproxW(dbl *mApprox, dbl *mResidule, ulng nPixelsPlain,
	ulng nColours, dbl *vWeights)
{
	/*
	% Calculate the approximation
		mApprox = mApprox - mResidule;
	% - - > > #
	*/

	Matrix::VAdd(mApprox,mResidule,-1.0/vWeights[0],nPixelsPlain);

	ulng iStartPlain = nPixelsPlain;

	for (ulng iColour = 1; iColour < nColours; iColour++)
	{
		Matrix::VAdd(&mApprox[iStartPlain],&mResidule[iStartPlain],
			-1.0/vWeights[iColour],nPixelsPlain);

		iStartPlain += nPixelsPlain;
	}

}


void OMPN::UpdateResidulePlain(dbl *vResidule, dbl *vImage, dbl *vOrth, dbl weight,
	ulng nElms)
{
		/* MATLAB
		% Calculate component of residule in mOrth
			mRemove = reshape( (mImageVec' * ...
				mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
		% - - > > #

		% Update the residule
			mResidule = mResidule - mRemove;
		% - - > > #    
		*/

		/* Calculate the inner product 
		mImage(:,:,i)*vOrth' */
		dbl innerProd = weight * Matrix::RealInnerProduct(vImage,vOrth,nElms);

		/* Remove the projection 
		mResidule = mResidule - innerProd*vOrth*/
		Matrix::VAdd(vResidule,vOrth,-innerProd,nElms);
}